import { WooCommerceSoftwareResult, WooCommerceSoftwareActivations } from "./types";
/**
 * A simple standalone implementation of [generate_key](https://woocommerce.com/document/software-add-on/#section-10) request
 * @param hostname The hostname/domain of the Wordpress/WooCommerce site
 * @param product_id The product ID representing the software product to generate license key for
 * @param email The user email to link the license key with
 * @param secret_key The secret key tied to the software product
 * @param order_id The order ID to link the license key with, default: null (_ignored_)
 * @param version Software version to use with the license key, default: null (_ignored_)
 * @param key_prefix A string to prepend the generated key with, default: null (_ignored_)
 * @param activations Number of activations allowable for the license key, default: 1
 * @returns Promise<WooCommerceSoftwareResult>
 */
export declare function generateKey(hostname: string, product_id: string, email: string, secret_key: string, order_id?: string | null, version?: string | null, key_prefix?: string | null, activations?: number, allowInsecure?: boolean): Promise<WooCommerceSoftwareResult>;
/**
 * A simple standalone implementation of [check](https://woocommerce.com/document/software-add-on/#section-14) request
 * @param hostname The hostname/domain of the Wordpress/WooCommerce site
 * @param product_id The product ID representing the software product to check the license key with
 * @param email The user email to check the license key with
 * @param license_key The license key to check for activations
 * @param timestamp A previously recorded timestamp/instance to test existing activations with, default: 0 (_ignored_)
 * @param platform The platform that is tied to the license activation, default: null (_ignored_)
 * @returns Promise<WooCommerceSoftwareResult>
 */
export declare function checkLicense(hostname: string, product_id: string, email: string, license_key: string, timestamp?: string | number, platform?: string | null, allowInsecure?: boolean): Promise<WooCommerceSoftwareResult>;
export declare class WooCommerceSoftwareAddOn {
    private hostname;
    private product_id;
    private email;
    private allowInsecure;
    /**
     * WooCommerceSoftwareAddOn Constructor
     * @param hostname The hostname/domain of the Wordpress/WooCommerce site
     * @param product_id The product ID representing the software product
     * @param email The user email to use for the API requests, default: null
     */
    constructor(hostname: string, product_id: string, email?: string | null, allowInsecure?: boolean);
    /**
     * A simple setter function to use if class is constructed without an email
     * @param email The user email to use for the API requests
     */
    setEmail(email: string): void;
    /**
     * A class method implementation of [generate_key](https://woocommerce.com/document/software-add-on/#section-10) request
     * @param secret_key The secret key tied to the software product
     * @param email The user email to link the license key with, default: null (_uses email set in constructor_)
     * @param order_id The order ID to link the license key with, default: null (_ignored_)
     * @param version Software version to use with the license key, default: null (_ignored_)
     * @param key_prefix A string to prepend the generated key with, default: null (_ignored_)
     * @param activations Number of activations allowable for the license key, default: 1
     * @returns Promise<WooCommerceSoftwareResult>
     */
    generateKey(secret_key: string, email?: string | null, order_id?: string | null, version?: string | null, key_prefix?: string | null, activations?: number): Promise<WooCommerceSoftwareResult>;
    /**
     * A class method implementation of [activation](https://woocommerce.com/document/software-add-on/#section-11) request
     * @param license_key The license key to activate for the current system
     * @param instance Pass to activate existing uses (previously deactivated), default: null (_ignored_)
     * @param platform The platform information that can be used to identify unique systems, default: null (_ignored_)
     * @returns Promise<WooCommerceSoftwareResult>
     */
    activateLicense(license_key: string, instance?: string | null, platform?: string | null): Promise<WooCommerceSoftwareResult>;
    /**
     * A class method implementation of [activation_reset](https://woocommerce.com/document/software-add-on/#section-12) request
     * @param license_key The license key to reset all activations in the Wordpress/WooCommerce site
     * @returns Promise<WooCommerceSoftwareResult>
     */
    resetLicense(license_key: string): Promise<WooCommerceSoftwareResult>;
    /**
     * A class method implementation of [deactivation](https://woocommerce.com/document/software-add-on/#section-13) request
     * @param license_key The license key to deactivate a single activation of
     * @param instance Pass to deactivate existing uses (previously activated), default: null (_ignored_)
     * @param activation_id Pass to deactivate existing uses (previously activated), default: null (_ignored_)
     * @returns Promise<WooCommerceSoftwareResult>
     */
    deactivateLicense(license_key: string, instance?: string | null, activation_id?: string | null): Promise<WooCommerceSoftwareResult>;
    /**
     * A class method implementation of [check](https://woocommerce.com/document/software-add-on/#section-14) request
     * @param license_key The license key to check for activations
     * @param timestamp A previously recorded timestamp/instance to test existing activations with, default: 0 (_ignored_)
     * @param platform The platform that is tied to the license activation, default: null (_ignored_)
     * @returns Promise<WooCommerceSoftwareResult>
     */
    checkLicense(license_key: string, timestamp?: string | number, platform?: string | null): Promise<WooCommerceSoftwareResult>;
    /**
     * An extended implementation [check](https://woocommerce.com/document/software-add-on/#section-14) request which returns a promise
     * @param license_key The license key to check for activations
     * @returns Promise<Array<WooCommerceSoftwareActivations> | null | undefined>
     *
     *      **Array<WooCommerceSoftwareActivations>** if check request was successful
     *
     *      **null** if check request was unsuccessful, or;
     *
     *      **undefined** if no email is set
     */
    getActivations(license_key: string): Promise<Array<WooCommerceSoftwareActivations> | null | undefined>;
    /**
     * A simple utility method to send API request to the WooCommerce Software Add-on
     * @param request Type of API request to be sent to the Wordpress/WooCommerce site
     * @param args Key-value containing the required and optional parameters for the request
     * @returns Promise<WooCommerceSoftwareResult>
     */
    private getRequest;
}
