"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WooCommerceSoftwareAddOn = exports.checkLicense = exports.generateKey = void 0;
const os_1 = __importDefault(require("os"));
const https_1 = __importDefault(require("https"));
const node_machine_id_1 = require("node-machine-id");
const systemInfo = `${os_1.default.hostname()} (${os_1.default.platform()}) ${os_1.default.cpus().map(cpu => {
    return cpu.model;
}).filter((cpu, index, self) => {
    return self.indexOf(cpu) === index;
})} (${(0, node_machine_id_1.machineIdSync)()})`;
const noEmailError = {
    success: false,
    code: undefined,
    headers: null,
    output: null,
    error: "No email set"
};
/**
 * A simple utility function to send API request to the WooCommerce Software Add-on
 * @param hostname The hostname/domain of the Wordpress/WooCommerce site
 * @param request Type of API request to be sent to the Wordpress/WooCommerce site
 * @param args Key-value containing the required and optional parameters for the request
 * @returns Promise<WooCommerceSoftwareResult>
 */
function getRequest(hostname, request, args, allowInsecure = false) {
    return __awaiter(this, void 0, void 0, function* () {
        let requestPath = "/woocommerce/?";
        let searchParams = new URLSearchParams(`wc-api=software-api&request=${request}`);
        Object.keys(args).forEach(key => {
            if (args[key]) {
                searchParams.append(key, String(args[key]));
            }
        });
        requestPath += searchParams.toString();
        return yield new Promise(resolve => {
            https_1.default.get({
                hostname: hostname,
                path: requestPath,
                agent: allowInsecure ? new https_1.default.Agent({
                    rejectUnauthorized: false
                }) : https_1.default.globalAgent
            }, (res) => {
                if (res.statusCode !== undefined && res.statusCode == 200) {
                    let data = "";
                    // A chunk of data has been received.
                    res.on('data', (d) => {
                        data += d;
                    });
                    // The whole response has been received. Print out the result.
                    res.on('end', () => {
                        let json = {};
                        try {
                            json = JSON.parse(data);
                        }
                        catch (e) {
                            resolve({
                                success: false,
                                code: res.statusCode,
                                headers: res.headers,
                                output: data,
                                error: e
                            });
                        }
                        let err = json;
                        if (err.error !== undefined) {
                            resolve({
                                success: false,
                                code: err.code,
                                headers: res.headers,
                                output: json,
                                error: `Code ${err.code}: ${err.error}`
                            });
                        }
                        resolve({
                            success: true,
                            code: res.statusCode,
                            headers: res.headers,
                            output: json,
                            error: "No error occurred"
                        });
                    });
                }
                else {
                    resolve({
                        success: false,
                        code: res.statusCode,
                        headers: res.headers,
                        output: null,
                        error: `Status code is ${res.statusCode}`
                    });
                }
            }).on('error', (e) => {
                resolve({
                    success: false,
                    code: 0,
                    headers: null,
                    output: null,
                    error: e
                });
            });
        });
    });
}
/**
 * A simple standalone implementation of [generate_key](https://woocommerce.com/document/software-add-on/#section-10) request
 * @param hostname The hostname/domain of the Wordpress/WooCommerce site
 * @param product_id The product ID representing the software product to generate license key for
 * @param email The user email to link the license key with
 * @param secret_key The secret key tied to the software product
 * @param order_id The order ID to link the license key with, default: null (_ignored_)
 * @param version Software version to use with the license key, default: null (_ignored_)
 * @param key_prefix A string to prepend the generated key with, default: null (_ignored_)
 * @param activations Number of activations allowable for the license key, default: 1
 * @returns Promise<WooCommerceSoftwareResult>
 */
function generateKey(hostname, product_id, email, secret_key, order_id = null, version = null, key_prefix = null, activations = 1, allowInsecure = false) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield getRequest(hostname, "generate_key", {
            secret_key: secret_key,
            email: email,
            product_id: product_id,
            order_id: order_id,
            version: version,
            key_prefix: key_prefix,
            activations: String(activations)
        }, allowInsecure);
    });
}
exports.generateKey = generateKey;
/**
 * A simple standalone implementation of [check](https://woocommerce.com/document/software-add-on/#section-14) request
 * @param hostname The hostname/domain of the Wordpress/WooCommerce site
 * @param product_id The product ID representing the software product to check the license key with
 * @param email The user email to check the license key with
 * @param license_key The license key to check for activations
 * @param timestamp A previously recorded timestamp/instance to test existing activations with, default: 0 (_ignored_)
 * @param platform The platform that is tied to the license activation, default: null (_ignored_)
 * @returns Promise<WooCommerceSoftwareResult>
 */
function checkLicense(hostname, product_id, email, license_key, timestamp = 0, platform = null, allowInsecure = false) {
    return __awaiter(this, void 0, void 0, function* () {
        let result = yield getRequest(hostname, "check", {
            email: email,
            license_key: license_key,
            product_id: product_id
        }, allowInsecure);
        if (result.success) {
            let activations = result.output.activations;
            let checkPlatform = platform != null ? platform : systemInfo;
            if (activations.length == 0) {
                result.success = false;
                result.error = "Software key is not yet activated";
            }
            else if (activations.find(activation => {
                // if timestamp is 0 (no timestamp was provided), instance is ignored
                // if activation_platform is empty string (no platform was recorded for the activation), platform is ignored
                return (timestamp == 0 || activation.instance == String(timestamp)) && ((activation.activation_platform == "") || (checkPlatform == activation.activation_platform));
            }) === undefined) {
                result.success = false;
                result.error = `No matching timestamp '${timestamp}' and/or platform '${checkPlatform}'`;
            }
        }
        return result;
    });
}
exports.checkLicense = checkLicense;
class WooCommerceSoftwareAddOn {
    /**
     * WooCommerceSoftwareAddOn Constructor
     * @param hostname The hostname/domain of the Wordpress/WooCommerce site
     * @param product_id The product ID representing the software product
     * @param email The user email to use for the API requests, default: null
     */
    constructor(hostname, product_id, email = null, allowInsecure = false) {
        this.hostname = hostname;
        this.product_id = product_id;
        this.email = email;
        this.allowInsecure = allowInsecure;
    }
    /**
     * A simple setter function to use if class is constructed without an email
     * @param email The user email to use for the API requests
     */
    setEmail(email) {
        this.email = email;
    }
    /**
     * A class method implementation of [generate_key](https://woocommerce.com/document/software-add-on/#section-10) request
     * @param secret_key The secret key tied to the software product
     * @param email The user email to link the license key with, default: null (_uses email set in constructor_)
     * @param order_id The order ID to link the license key with, default: null (_ignored_)
     * @param version Software version to use with the license key, default: null (_ignored_)
     * @param key_prefix A string to prepend the generated key with, default: null (_ignored_)
     * @param activations Number of activations allowable for the license key, default: 1
     * @returns Promise<WooCommerceSoftwareResult>
     */
    generateKey(secret_key, email = null, order_id = null, version = null, key_prefix = null, activations = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            let _email = email != null ? email : this.email;
            if (_email == null)
                return noEmailError;
            return generateKey(this.hostname, this.product_id, _email, secret_key, order_id, version, key_prefix, activations, this.allowInsecure);
        });
    }
    /**
     * A class method implementation of [activation](https://woocommerce.com/document/software-add-on/#section-11) request
     * @param license_key The license key to activate for the current system
     * @param instance Pass to activate existing uses (previously deactivated), default: null (_ignored_)
     * @param platform The platform information that can be used to identify unique systems, default: null (_ignored_)
     * @returns Promise<WooCommerceSoftwareResult>
     */
    activateLicense(license_key, instance = null, platform = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.email == null)
                return noEmailError;
            return yield this.getRequest("activation", {
                email: this.email,
                license_key: license_key,
                product_id: this.product_id,
                instance: instance,
                platform: platform != null ? platform : systemInfo
            });
        });
    }
    /**
     * A class method implementation of [activation_reset](https://woocommerce.com/document/software-add-on/#section-12) request
     * @param license_key The license key to reset all activations in the Wordpress/WooCommerce site
     * @returns Promise<WooCommerceSoftwareResult>
     */
    resetLicense(license_key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.email == null)
                return noEmailError;
            return yield this.getRequest("activation_reset", {
                email: this.email,
                license_key: license_key,
                product_id: this.product_id
            });
        });
    }
    /**
     * A class method implementation of [deactivation](https://woocommerce.com/document/software-add-on/#section-13) request
     * @param license_key The license key to deactivate a single activation of
     * @param instance Pass to deactivate existing uses (previously activated), default: null (_ignored_)
     * @param activation_id Pass to deactivate existing uses (previously activated), default: null (_ignored_)
     * @returns Promise<WooCommerceSoftwareResult>
     */
    deactivateLicense(license_key, instance = null, activation_id = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.email == null)
                return noEmailError;
            return yield this.getRequest("deactivation", {
                email: this.email,
                license_key: license_key,
                product_id: this.product_id,
                instance: instance,
                activation_id: activation_id
            });
        });
    }
    /**
     * A class method implementation of [check](https://woocommerce.com/document/software-add-on/#section-14) request
     * @param license_key The license key to check for activations
     * @param timestamp A previously recorded timestamp/instance to test existing activations with, default: 0 (_ignored_)
     * @param platform The platform that is tied to the license activation, default: null (_ignored_)
     * @returns Promise<WooCommerceSoftwareResult>
     */
    checkLicense(license_key, timestamp = 0, platform = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.email == null)
                return noEmailError;
            return checkLicense(this.hostname, this.product_id, this.email, license_key, timestamp, platform != null ? platform : systemInfo, this.allowInsecure);
        });
    }
    /**
     * An extended implementation [check](https://woocommerce.com/document/software-add-on/#section-14) request which returns a promise
     * @param license_key The license key to check for activations
     * @returns Promise<Array<WooCommerceSoftwareActivations> | null | undefined>
     *
     *      **Array<WooCommerceSoftwareActivations>** if check request was successful
     *
     *      **null** if check request was unsuccessful, or;
     *
     *      **undefined** if no email is set
     */
    getActivations(license_key) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.email == null)
                return undefined;
            return yield checkLicense(this.hostname, this.product_id, this.email, license_key, "", "", this.allowInsecure).then(value => {
                if (value.success)
                    return value.output.activations;
                return null;
            });
        });
    }
    /**
     * A simple utility method to send API request to the WooCommerce Software Add-on
     * @param request Type of API request to be sent to the Wordpress/WooCommerce site
     * @param args Key-value containing the required and optional parameters for the request
     * @returns Promise<WooCommerceSoftwareResult>
     */
    getRequest(request, args) {
        return __awaiter(this, void 0, void 0, function* () {
            return getRequest(this.hostname, request, args, this.allowInsecure);
        });
    }
}
exports.WooCommerceSoftwareAddOn = WooCommerceSoftwareAddOn;
